/*-----------------------------------------------------------------------------
 * Name:    GLCD_Config.h
 * Purpose: Graphic LCD interface configuration file for EFM32GG-DK3750 Development Kit
 * Rev.:    1.0.0
 *----------------------------------------------------------------------------*/

/* Copyright (c) 2013 - 2014 ARM LIMITED

   All rights reserved.
   Redistribution and use in source and binary forms, with or without
   modification, are permitted provided that the following conditions are met:
   - Redistributions of source code must retain the above copyright
     notice, this list of conditions and the following disclaimer.
   - Redistributions in binary form must reproduce the above copyright
     notice, this list of conditions and the following disclaimer in the
     documentation and/or other materials provided with the distribution.
   - Neither the name of ARM nor the names of its contributors may be used
     to endorse or promote products derived from this software without
     specific prior written permission.
   *
   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
   AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
   IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
   ARE DISCLAIMED. IN NO EVENT SHALL COPYRIGHT HOLDERS AND CONTRIBUTORS BE
   LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
   CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
   SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
   INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
   CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
   ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
   POSSIBILITY OF SUCH DAMAGE.
   ---------------------------------------------------------------------------*/

#ifndef __GLCD_CONFIG_H
#define __GLCD_CONFIG_H

/*---------------------- Graphic LCD physical definitions --------------------*/
#define GLCD_SIZE_X     320             /* Screen size X (in pixels) */
#define GLCD_SIZE_Y     240             /* Screen size Y (in pixels) */
#define GLCD_BPP        24              /* Bits per pixel            */

#define GLCD_WIDTH      GLCD_SIZE_X     /* Screen Width  (in pixels) */
#define GLCD_HEIGHT     GLCD_SIZE_Y     /* Screen Height (in pixels) */

/*---------------------- Graphic LCD color definitions -----------------------*/
/* Color coding (24-bit): 
     23..16 = R7..0 (Red)
     15.. 8 = G7..0 (Green)
      7.. 0 = B7..0 (Blue)
*/

/* GLCD RGB color definitions                            */
#define GLCD_COLOR_BLACK      0x000000  /*   0,   0,   0 */
#define GLCD_COLOR_NAVY       0x000080  /*   0,   0, 128 */
#define GLCD_COLOR_DARK_GREEN 0x008000  /*   0, 128,   0 */
#define GLCD_COLOR_DARK_CYAN  0x008080  /*   0, 128, 128 */
#define GLCD_COLOR_MAROON     0x800000  /* 128,   0,   0 */
#define GLCD_COLOR_PURPLE     0x800080  /* 128,   0, 128 */
#define GLCD_COLOR_OLIVE      0x808000  /* 128, 128,   0 */
#define GLCD_COLOR_LIGHT_GREY 0xC0C0C0  /* 192, 192, 192 */
#define GLCD_COLOR_DARK_GREY  0x808080  /* 128, 128, 128 */
#define GLCD_COLOR_BLUE       0x0000FF  /*   0,   0, 255 */
#define GLCD_COLOR_GREEN      0x00FF00  /*   0, 255,   0 */
#define GLCD_COLOR_CYAN       0x00FFFF  /*   0, 255, 255 */
#define GLCD_COLOR_RED        0xFF0000  /* 255,   0,   0 */
#define GLCD_COLOR_MAGENTA    0xFF00FF  /* 255,   0, 255 */
#define GLCD_COLOR_YELLOW     0xFFFF00  /* 255, 255, 0   */
#define GLCD_COLOR_WHITE      0xFFFFFF  /* 255, 255, 255 */

#endif /* __GLCD_CONFIG_H */
