/*----------------------------------------------------------------------------
 * Name:    Blinky.c
 * Purpose: LED Flasher for EFM32GG-DK3750 Development Kit
 *----------------------------------------------------------------------------
 * This file is part of the uVision/ARM development tools.
 * This software may only be used under the terms of a valid, current,
 * end user licence from KEIL for a compatible version of KEIL software
 * development tools. Nothing else gives you the right to use this software.
 *
 * This software is supplied "AS IS" without warranties of any kind.
 *
 * Copyright (c) 2004-2014 Keil - An ARM Company. All rights reserved.
 *----------------------------------------------------------------------------*/

#include "cmsis_os.h"
#include "em_device.h"
#include "em_chip.h"
#include "em_cmu.h"
#include "bsp.h"
#include "Board_LED.h"

/*----------------------------------------------------------------------------
  Main function
 *----------------------------------------------------------------------------*/
int main (void) {
  int32_t max_num = LED_GetCount() - 1;
  int32_t num = 0;
  int32_t dir = 1;

  /* Chip errata */
  CHIP_Init();

  /* HFXO (High frequency crystal oscillator) is 48MHz */
  CMU_OscillatorEnable (cmuOsc_HFXO, true, true);
  CMU_ClockSelectSet   (cmuClock_HF,    cmuSelect_HFXO);
  CMU_ClockDivSet      (cmuClock_HFPER, cmuClkDiv_1);

  SystemCoreClockUpdate();

  LED_Initialize();                         /* LED Initialization             */

  while (1) {
    LED_On(num);                            /* Turn specified LED on          */
    osDelay(500);                           /* Wait 500ms                     */
    LED_Off(num);                           /* Turn specified LED off         */
    osDelay(500);                           /* Wait 500ms                     */

    num += dir;                             /* Change LED number              */
    if (dir == 1 && num == max_num) {
      dir = -1;                             /* Change direction to down       */
    }
    else if (num == 0) {
      dir =  1;                             /* Change direction to up         */
    }
  }
}
